// MyCustomDraw.cpp: implementation of the CMyCustomDraw class.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "CustomDrawing.h"
#include "MyCustomDraw.h"
#include "Order.h"
#include "CustomDrawingDoc.h"

#include "Dapfor/GUI/MsImg32.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMyCustomDraw::CMyCustomDraw(bool autoDelete) 
	: CCustomDraw(autoDelete)
	, m_EnableColoring(true)
	, m_EnableIconsPainting(true)
{
	//The method CImageList::Create(...) does not permit to load 32-bit color image
	//To show full-colored images in cells of the grid, you can use the helper from the Dapfor::GUI library
	Dapfor::GUI::CMsImg32::Load32BitImage(m_Bullets, IDB_BULLETS, true, RGB(255, 255, 255));
}

CMyCustomDraw::~CMyCustomDraw()
{

}


void CMyCustomDraw::DrawLine(Dapfor::GUI::CGridLine& line, const Dapfor::GUI::CPaintContext& paintContext, UINT paintFilter)
{
	const Dapfor::Common::CDataObject* pDO = line.GetDataObject();
	if(pDO && pDO->GetLong(FidObjectType) == Document)
	{
		//You have an access to the CDC and at any moment you can design your own content.
		//The next example demonstrates how to merge all cells in the line and to draw a custom text.
		

		//Prevent from drawing the text and images in cells.
		paintFilter &= paintFilter^(Dapfor::GUI::drawText|Dapfor::GUI::drawImage);
		
		//Fill the background
		BaseClass::DrawLine(line, paintContext, paintFilter);
		
		//Save old parameters of the CDC and set the transparent mode
		int oldMode = paintContext.GetDC().GetBkMode();
		paintContext.GetDC().SetBkMode(TRANSPARENT);
		
		//Create some string.
		CString s; 
		s.Format(_T("This non-scrollable line represents the document with %d orders"), 
			     pDO->GetLong(CCustomDrawingDoc::FidOrdersCount));
		
		//Here we have all the information to draw the text: Bounds, text etc... 
		CRect rc(&line.GetPaintInfo().GetVisibleRect());
		paintContext.GetDC().DrawText(s, rc, DT_CENTER|DT_END_ELLIPSIS);
		
		//Restore the original mode
		paintContext.GetDC().SetBkMode(oldMode);
	}
	else
	{
		//Default behavior
		BaseClass::DrawLine(line, paintContext, paintFilter);
	}
}

void CMyCustomDraw::DrawCell(Dapfor::GUI::CGridCell& cell, const Dapfor::GUI::CPaintContext& paintContext, UINT paintFilter)
{
	//This variable should be visible by the base class
	CFont font;
	const Dapfor::Common::CDataObject* pDO = cell.GetGridLine().GetDataObject();
	
	//Customize the font for each cell
	CFont* initialFont = cell.GetPaintInfo().GetFont();
	if(initialFont && m_EnableColoring)
	{
		//Get the current font
		LOGFONT logFont;
		initialFont->GetLogFont(&logFont);

		//Change the font's size for the 'Product' column
		if(cell.GetColumn() != 0 && (cell.GetColumn()->GetFid() == COrder::FidProduct))
		{
			logFont.lfHeight = -MulDiv(12, GetDeviceCaps(paintContext.GetDC(), LOGPIXELSY), 72);
		}
		if(cell.GetColumn() != 0 && (cell.GetColumn()->GetFid() == COrder::FidUnitPrice))
		{
			//Change the font family
			TCHAR name[] = _T("Arial");
			memcpy(logFont.lfFaceName, name, sizeof(name));
			logFont.lfItalic = 1;
			logFont.lfWeight = 600;
		}
		//Draw all payed orders with the bold font
		if(pDO && (pDO->GetLong(COrder::FidStatus) == COrder::Payed))
		{
			logFont.lfWeight = 600;
		}
		
		font.CreateFontIndirect(&logFont);
		cell.GetPaintInfo().SetFont(&font);

	}
	
	//Set a custom color for all the line
	if(cell.GetColumn() && pDO && m_EnableColoring)
	{
		UINT color = cell.GetPaintInfo().GetForeColor();
		switch(pDO->GetLong(COrder::FidStatus))
		{
			case COrder::Ordered:	color = RGB(202, 0, 2); break;
			case COrder::Payed:		color = RGB(0, 66, 118); break;
			case COrder::Refunded:	color = RGB(0, 153, 0); break;
			case COrder::Cancelled:	color = RGB(100, 27, 27); break;
		}
		cell.GetPaintInfo().SetForeColor(color);
	}


	//Set an image for the 'Unit price' column
	if( cell.GetColumn() && 
	   (cell.GetColumn()->GetFid() == COrder::FidUnitPrice) && 
	    pDO &&
		m_EnableIconsPainting)
	{
		cell.SetImageList(&m_Bullets);
		
		//Select some index in the image list
		int iconIndex = ((GetTickCount() % 1740 - 30*cell.GetGridLine().GetRow()) / 300) % 6;
		cell.SetImageId(iconIndex);
		cell.SetStretchImage(false);

		//request for the next drawing routine for the given cell in order to change the icon in few milliseconds
		cell.RequestNextPaint(30);
	}

	
	BaseClass::DrawCell(cell, paintContext, paintFilter);
}


/// Calculates an optimal width of a cell taking into account a font, hierarchy, text size...
int CMyCustomDraw::GetOptimalCellWidth(Dapfor::GUI::CGridCell& cell, const Dapfor::GUI::CPaintContext& paintContext, UINT paintFilter) const
{
	//This variable should be visible by the base class
	CFont font;
	long additionalWidth = 0;
	
	//Customize the font for each cell
	CFont* initialFont = cell.GetPaintInfo().GetFont();
	if(initialFont && m_EnableColoring)
	{
		//Get the current font
		LOGFONT logFont;
		initialFont->GetLogFont(&logFont);

		logFont.lfWeight = 600;
		
		//Change the font's size for the 'Product' column
		if(cell.GetColumn() != 0 && (cell.GetColumn()->GetFid() == COrder::FidProduct))
		{
			logFont.lfHeight = -MulDiv(12, GetDeviceCaps(paintContext.GetDC(), LOGPIXELSY), 72);
		}
		if(cell.GetColumn() != 0 && (cell.GetColumn()->GetFid() == COrder::FidCount))
		{
			//Take into account a width of icon, shown in the column
			//additionalWidth = cell.GetPaintInfo().GetVirtualRect().Height();
		}		

		if(cell.GetColumn() != 0 && (cell.GetColumn()->GetFid() == COrder::FidUnitPrice))
		{
			TCHAR name[] = _T("Arial");
			memcpy(logFont.lfFaceName, name, sizeof(name));
			logFont.lfItalic = 1;
		}		
		
		font.CreateFontIndirect(&logFont);
		cell.GetPaintInfo().SetFont(&font);
	}

	//Take into account a width of icon, shown in the column
	if(m_EnableIconsPainting && cell.GetColumn() != 0 && (cell.GetColumn()->GetFid() == COrder::FidUnitPrice))
	{
		additionalWidth = cell.GetPaintInfo().GetVirtualRect().Height();
	}


	//The base class will do the computing...
	return additionalWidth + BaseClass::GetOptimalCellWidth(cell, paintContext, paintFilter);
}

int CMyCustomDraw::GetOptimalRowHeight(CFont* font, CDC& dc) const
{
	//return the height
	return 16;
}

void CMyCustomDraw::EnableColoring(bool enable)
{
	m_EnableColoring = enable;
}
bool CMyCustomDraw::IsColoringEnabled() const
{
	return m_EnableColoring;
}

void CMyCustomDraw::EnableIconsPainting(bool enable)
{
	m_EnableIconsPainting = enable;
}
bool CMyCustomDraw::IsIconsPaintingEnabled() const
{
	return m_EnableIconsPainting;
}
