// Folder.cpp: implementation of the CFolderData class.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "FolderData.h"
#include "Types.h"
#include "FileData.h"

#include <Dapfor/Common/Lock.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


//Field map implementation. Here you declare the properties of the class that will be accessible from the grid.
//Each property has an unique identifier, and optionally may have a name, Get-function, Set-function and a format 
//to convert value, returned by the Get- function to string and vice-versa.
DF_BEGIN_FIELD_MAP(CFolderData)
    DF_MFC_STRING_ID(FidName,      _T("Name"),        &CFolderData::GetName,        0, 0)
    DF_MFC_STRING_ID(FidFullName,  _T("Full Name"),   &CFolderData::GetFullName,    0, 0)
    DF_INT64_ID(FidSize,           _T("Size"),        &CFolderData::GetSize,        0, 0)
    DF_LONG_ID(FidFilesCount,       _T("Files Count"), &CFolderData::GetFilesCount,  0, 0)
    DF_BOOL_ID(FidIsPopulating,    _T("Populating"),  &CFolderData::IsPopulating,   0, 0)
    DF_LONG_ID(FidItemType,        _T("Type"),        &CFolderData::GetType,        0, 0)
DF_END_FIELD_MAP()


//The constructor       
CFolderData::CFolderData(const CString& name, CFolderData* parent) 
    : m_Name(name)
    , m_Parent(parent)
    , m_FilesSize(0)
    , m_IsPopulating(false)
{
}

//The destructor
CFolderData::~CFolderData()
{
    //Delete all attached files
    for(Files::iterator itFile = m_Files.begin(); itFile != m_Files.end(); ++itFile)
    {
        //(*itFile)->RemoveSink(this);
        delete *itFile;
    }

    //Delete all attached folders
    for(Folders::iterator itFolder = m_Folders.begin(); itFolder != m_Folders.end(); ++itFolder)
    {
        (*itFolder)->RemoveSink(this);
        delete *itFolder;
    }

    
}

const CFolderData* CFolderData::GetParent() const
{
    return m_Parent;
}

CString CFolderData::GetName() const
{
    //Get the folder name
    return m_Name;
}

CString CFolderData::GetFullName() const
{
    //Build the full folder name
    return m_Parent != 0 
                ? m_Parent->GetFullName() + CString("\\") + m_Name 
                : m_Name;
}


void CFolderData::AddSubFolder(CFolderData* child)
{
    //Add a new folder
    if(child != 0)
    {
        //Protect the m_Folders by the critical section
        Dapfor::Common::CLock lock(m_Synchro);

        m_Folders.push_back(child);
        
        //Register the callback to get notifications.
        child->AddSink(this);
    }
    
    NotifyUpdate(FidSize);
    NotifyUpdate(FidFilesCount);
}

void CFolderData::AddFile(CFileData* file)
{
    //Add a new file
    if(file != 0)
    {
        //Protect the m_Files & m_FilesSize by the critical section
        Dapfor::Common::CLock lock(m_Synchro);

        m_FilesSize += file->GetSize();
        m_Files.push_back(file);
    }
}

bool CFolderData::IsPopulating() const
{
    return m_IsPopulating;
}

long CFolderData::GetType() const
{
    return FolderData;
}


void CFolderData::BeginPopulate()
{
    m_IsPopulating = true;
    NotifyUpdate(Dapfor::Common::allFields);
}

void CFolderData::EndPopulate()
{
    m_IsPopulating = false;
    NotifyUpdate(Dapfor::Common::allFields);
}

__int64 CFolderData::GetSize() const
{
    //Protect the m_Folders by the critical section
    Dapfor::Common::CLock lock(m_Synchro);
    
    //Compute the size of all sub folders
    __int64 size = 0;
    for(Folders::const_iterator it = m_Folders.begin(); it != m_Folders.end(); ++it)
    {
        size += (*it)->GetSize();

    }

    //The size of the folder is the sum of sizes of all files and all subfolders.
    return size + m_FilesSize;
}

long CFolderData::GetFilesCount() const
{
    //Protect the m_Files
    Dapfor::Common::CLock lock(m_Synchro);
    long count = 0;
    for(Folders::const_iterator it = m_Folders.begin(); it != m_Folders.end(); ++it)
    {
        count += (*it)->GetFilesCount();
    }

    count += m_Files.size();
    return count;
}

CFolderData::Folders& CFolderData::GetFolders()
{
    return m_Folders;
}


void CFolderData::OnUpdate(Dapfor::Common::FID fid, const Dapfor::Common::CDataObject* dataObject)
{
    //This function is called when a new file has been added to one of subfolders

    //Normally, we can do nothing, just will send a notification. Note that there are at last two listeners: 
    // 1. The parent folder
    // 2. The grid. 
    // When the grid receives a notification, it updates the text, shown in the cell and also can 
    // filter, scroll and highlight the item. If this item was inserted into various grids, then
    // each grid will perform the same operations automatically.
    
    switch(fid)
    {
        case FidSize:
        case FidFilesCount:
            NotifyUpdate(fid);
            break;
    }
}
