/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/Types.h>
#include <Dapfor/GUI/GUI_Export.h>

namespace Dapfor
{

//Forward declarations
namespace Common
{
    class CFormat;
}

namespace GUI
{

class ICellPaintFormat;
class IColumnPaintFormat;

//////////////////////////////////////////////////////////////////////////////////////////////
/// \brief This class implements a column of the header
///
/// The given class contains the necessary data for displaying columns in the grid. 
/// The grid works with C++ objects of practically any classes. The only condition lies in the fact that these 
/// classes should inherit from Common::CDataObject. The basic application of the given interface is to give 
/// the opportunity of receiving as unformatted so formatted values through data fields identifiers. 
/// Unformatted data are the values of the basic types (long, double...) returned by data object functions. 
/// The given functions can be called through integer identifiers, and then the received unformatted value can be 
/// converted into a string and displayed in the corresponding grid cell. 
/// Each column contains an identifier and if it corresponds to a data object field identifier the value of this field 
/// will be displayed in a grid  cell through a system of formats. As mentioned, primarily data objects return 
/// unformatted value that cannot be displayed directly on the screen. The way of displaying depends on a text format 
/// which converts the given value into a string. The type of the chosen format is defined by the following: 
/// First the format is searched in a grid column. If it is not found, the search occurs in the data fields table 
/// (Common::CFieldMap) of the corresponding object and if the format is not found yet, the default format, returned by 
/// Common::CFormat::GetDefaultFormat() function is taken. A class object of a column contains such parameters as a width 
/// of a column, an alignment of a text including an automatically defined one, depending on a field type, 
/// indexes of pictures for grid cells and for a header. A column can store graphic formats for an individual portrayal 
/// of cells within the bounds of a column, and also in a header. 
///
/// Also it's possible to organize a subscription for recieving  notifications from CColumn class object. 
/// In the case of inner data changes, all subscribers recieve notifications. While inserting a column in a header, 
/// there is an automatic subscription to receive the events which are later transfered in the grid. Thus, in the case 
/// of calling class Set-methods of a column, the grid updates contents of the screen in the appropriate way. 
///
/// It is also important to note, that practically all the functions of CColumn class are threadsafe and can be called 
/// from any tread. All events are sent in such a way, that at the moment of their sending synchronizing objects of 
/// such type as critical section or mutex are not occupied.
///
/// 
/// \class  CColumn Column.h Dapfor/GUI/Column.h
/// \file Column.h
//////////////////////////////////////////////////////////////////////////////////////////////
class DF_GUI_API CColumn : public CObject
{
public:
    ///Text alignment
    enum Alignment
    {
        Auto,       //!< alignment depends on the field's type (long, string, double, etc...)
        Left,       //!< text is aligned to the left of the cell
        Right,      //!< text is aligned to the right of the cell
        Center,     //!< text is aligned in the center of the cell
    };

    /// Notification type
    enum NotificationType
    {
        NameChanged,             //!< Name of the column is changed
        WidthChanged,            //!< Width of the column is changed
        AlignmentChanged,        //!< Alignment type is changed
        DescriptionChanged,      //!< Column description is changed
        CellPaintFormatChanged,  //!< Cell painting format is changed
        ColumnPaintFormatChanged,//!< Column painting format is changed
        TextFormatChanged,       //!< Text format that converts non-formatted data to strings is changed
        CellImageChanged,        //!< Image list or index of the icon drawn in the cell is changed
        ColumnImageChanged,      //!< Image list or index of the icon drawn in the column is changed
        VisibleChanged,          //!< Visibility of the column is changed
        EditableChanged,         //!< Editable flag is changed

        ColumnDelete,            //!< Is sent in the destructor of CColumn object
    }; 

    /// Interface to receive notifications about column state modification
    struct INotify
    {
        virtual void OnColumnModified(const CColumn* column, NotificationType type) = 0;
    };

public:
    // Constructors & destructors
    CColumn(Common::FID            fid, 
            const TString&              name, 
            int                         width, 
            Alignment                   alignment = Auto, 
            bool                        editable = false,
            int                         cellImageId = -1,
            int                         columnImageId = -1,
            const Common::CFormat* textFormat = 0, 
            const ICellPaintFormat*     cellPaintFormat = 0, 
            const IColumnPaintFormat*   columnPaintFormat = 0, 
            const TString&              description = _T(""));

    CColumn(const CColumn& column);
    virtual ~CColumn();

    //Get- methods

    Common::FID            GetFid() const;              //!< Gets the field's identifier
    const TString&              GetName() const;             //!< Gets a column name
    int                         GetWidth() const;            //!< Gets a column width
    Alignment                   GetAlignment() const;        //!< Gets text alignment 
    bool                        IsEditable() const;          //!< Indicates whether the cells belonged to this column can be editable.
    bool                        IsVisible() const;           //!< Indicates whether the column is visible.
    int                         GetCellImageId() const;      //!< Gets Zero-based index of the image to draw an icon in the cell.
    int                         GetColumnImageId() const;    //!< Gets Zero-based index of the image to draw an icon in the column. 
    const Common::CFormat* GetTextFormat() const;       //!< Gets custom textual format to convert the non-formatted value to the formatted string
    const ICellPaintFormat*     GetCellPaintFormat() const;  //!< Gets the format to draw the cells belonged to this column.
    const IColumnPaintFormat*   GetColumnPaintFormat() const;//!< Gets custom format to draw this column.
    TString                     GetDescription() const;      //!< Gets the column description
    
    //Set- methods
    void                        SetName(const TString& name);                     //!< Sets a new column name
    bool                        SetWidth(int columnWidth);                        //!< Sets a new column width
    void                        SetAlignment(Alignment alignment);                //!< Sets a new text alignment 
    void                        SetEditable(bool bEditable);                      //!< Allows cells to be editable
    void                        SetVisible(bool bVisible);                        //!< Shows or hides a column
    void                        SetCellImageId(int imageId);                      //!< Sets a new zero-based index of the image to draw an icon in the cell.
    void                        SetColumnImageId(int imageId);                    //!< Sets a new zero-based index of the image to draw an icon in the column. 
    const Common::CFormat* SetTextFormat(const Common::CFormat* format);//!< Sets a new textual format to convert the non-formatted value to the formatted string
    const ICellPaintFormat*     SetCellPaintFormat(const ICellPaintFormat* cellPaintFormat);      //!< Sets a new format to draw the cells belonged to this column.
    const IColumnPaintFormat*   SetColumnPaintFormat(const IColumnPaintFormat* columnPaintFormat);//!< Sets a new format to draw this column.
    void                        SetDescription(const TString& description);       //!< Sets a new column description

    //Subscription to events
    void                        Subscribe(INotify* callBack);  //!< Subscribes for events 
    void                        Unsubscribe(INotify* callBack);//!< Unsubscribes from events 

    //Serialization
    virtual void                Serialize(CArchive& ar);       //!< Serializes or deserializes data to/from the archive 
    static int                  GetSerializeVersion();         //!< Current serialization version.

    CColumn& operator = (const CColumn&);

protected:
    CColumn();

    /// Sends specified notification to all subscribers
    void                        Notify(NotificationType type);

private:
    DECLARE_SERIAL(CColumn)

private:
    struct Impl;
    Impl* m_Impl;
};


}

}//namespace Dapfor

