/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/GUI_Export.h>
#include <Dapfor/GUI/Types.h>
#include <Dapfor/GUI/IEditInPlace.h>

namespace Dapfor
{
namespace GUI
{
class CGrid;
class CHeader;
class CColumn;
class CDragSource;
class CDropTarget;

namespace Notifications
{

/*! \page Notifications "Notifications"

The grid is capable to inform its owners about some events. 
For example about an addition of a new line, selection or focus changes, etc. The given information 
is sent in the form of WM_NOTIFY message where one of the structures, containing the detailed information 
about an event, acts as lParam. Below there is a list of messages, and also the corresponding structures 
containing the information on the event. 
For increasing the grid productivity it is possible to block sending events by call of 
CGrid::AllowNotifications() function.

List of available notifications:
 - NM_GRIDITEM
 - NM_GRIDDRAGDROP
 - NM_GRIDEDIT
 - NM_GRIDCOLUMN
 - NM_GRIDSORT

*/



/// Notification type
enum NotificationType
{
    AddItem,            //!< A new item has been added to the grid
    RemoveItem,         //!< An item has been removed from the grid
    MoveItem,           //!< An item has been displaced in the grid
    AttachItem,         //!< Already unattached item has been attached to the grid
    DetachItem,         //!< An item has been detached from the grid 

    FocusChanged,       //!< A new item is focused in the grid (or focus was removed)
    SelectionChanged,   //!< Selection has been changed
    ExpansionChanged,   //!< An item has been expanded or collapsed
    FilterChanged,      //!< After item updating it became filtered or vice versa
    VisibilityChanged,  //!< After item updating or collapsing/expanding the item changed its visibility 
    UpdateItem,         //!< The item was updated by ISink interface or by the CGrid::Update(...) call
    
    BeginDrag,          //!< Start of the drag & drop
    EndDrag,            //!< End of the the drag & drop
    BeginDrop,          //!< Start of the dropping data from the clipboard 
    EndDrop,            //!< End of the dropping data from the clipboard 
    BeginEdit,          //!< Start of edit in place
    EndEdit,            //!< End of edit in place
    
    AddColumn,          //!< A new column has been added
    RemoveColumn,       //!< A column has been removed
    MoveColumn,         //!< A column in the header has been displaced
    ResizeColumn,       //!< A column in the header has been resized
    UpdateColumn,       //!< Any parameter of the column has been changed

    SortChanged,        //!< Sorting has been modified
};

/// Item states
enum ItemState
{
    ItemSelected        = 0x01, //!< The item is selected
    ItemFocused         = 0x02, //!< The item is focused
    ItemVisible         = 0x04, //!< The item is visible (not filtered and not hidden by its parents)
    ItemExpanded        = 0x08, //!< The item is expanded
    ItemFiltered        = 0x10, //!< The item is filtered
    ItemAttached        = 0x20, //!< The item is actually attached to the grid
};

enum ColumnState
{
    ColumnVisible       = 0x01, //!< The column is visible
    ColumnFixed         = 0x02, //!< The column is fixed
    ColumnSorted        = 0x04, //!< The column has sorting
};

}//namespace Notifications

///\brief Sent by the grid to notify its parent window about item updating.
///This notification is sent in the form of WM_NOTIFY message. 
struct DF_GUI_API NM_GRIDITEM { 
    NMHDR                           hdr;        //!< NMHDR structure that contains information about this notification message. 
    Notifications::NotificationType type;       //!< Notificaition type
    CGrid*                          grid;       //!< The grid which sends this notification
    UINT                            state;      //!< Combination of GUI::Notifications::ItemState enumeration
    HITEM                           item;       //!< Handle in the grid
    Common::CDataObject*       pDO;        //!< The data object, associated with a handle
    ContextType                     context;    //!< The grid context to which a handle is related
    int                             row;        //!< Index of a line in the context
};

///\brief Sent by the grid to notify its parent window about drag & drop processing.
///This notification is sent in the form of WM_NOTIFY message. 
struct DF_GUI_API NM_GRIDDRAGDROP { 
    NMHDR                           hdr;        //!< NMHDR structure that contains information about this notification message. 
    Notifications::NotificationType type;       //!< Notificaition type
    CGrid*                          grid;       //!< The grid which sends this notification
    CDragSource*                    dragSource; //!< Source of a content
    CDropTarget*                    dropTarget; //!< Location where a clipboard content is to be dropped
    DROPEFFECT                      effect;     //!< Drop effect. One of the next values: DROPEFFECT_COPY, DROPEFFECT_MOVE, DROPEFFECT_LINK, DROPEFFECT_NONE
};



///\brief Sent by the grid to notify its parent window about edit in place processing.
///This notification is sent in the form of WM_NOTIFY message. 
struct DF_GUI_API NM_GRIDEDIT { 
    NMHDR                           hdr;        //!< NMHDR structure that contains information about this notification message. 
    Notifications::NotificationType type;       //!< Notificaition type
    CGrid*                          grid;       //!< The grid which sends this notification
    HITEM                           item;       //!< Handle in the grid
    Common::CDataObject*       pDO;        //!< The data object, associated with a handle
    ContextType                     context;    //!< The grid context to which a handle is related
    int                             row;        //!< Index of a line in the context
    CColumn*                        column;     //!< A column
    CWnd*                           control;    //!< Edit in place control if it exists
    IEditInPlace::StartReason       startReason;//!< Edit in place start reason
    IEditInPlace::StopReason        stopReason; //!< Edit in place stop reason
};



///\brief Sent by the grid to notify its parent window about column changing.
///This notification is sent in the form of WM_NOTIFY message. 
struct DF_GUI_API NM_GRIDCOLUMN { 
    NMHDR                           hdr;    //!< NMHDR structure that contains information about this notification message. 
    Notifications::NotificationType type;   //!< Notificaition type
    CGrid*                          grid;   //!< The grid which sends this notification
    CHeader*                        header; //!< A header of the grid
    UINT                            state;  //!< Combination of GUI::Notifications::ColumnState enumeration
    CColumn*                        column; //!< The column where it is appropriate
};


///\brief Sent by the grid to notify its parent window about sorting changing.
///This notification is sent in the form of WM_NOTIFY message. 
struct DF_GUI_API NM_GRIDSORT { 
    NMHDR                           hdr;    //!< NMHDR structure that contains information about this notification message. 
    Notifications::NotificationType type;   //!< Notificaition type
    CGrid*                          grid;   //!< The grid which sent this notification
    CHeader*                        header; //!< A header of the grid
};



}//namespace GUI

}//namespace Dapfor

